######################################################################################
#  Supplementary Materials.   Dec. 31, 2016                           																	         #
#  R code for Example 1 in Afzal et al. (2016) for the journal of Statisical Modelling titled                         #
# "Estimation of Partly Linear Additive Hazards Model with Left Truncated and Right Censored Data";    #
######################################################################################

# Generate data set for the Partly Linear Additive Hazards Model
# Model: \lambda(t;z1,z2)=5+(0.3z1-0.5z2)+\phi1(W1)+\phi(W2)
# z1~runif(n,1,4)-2.5
# z2~rbinom(n,.5)-.5
# W1~runif(3,9)
# W2~runif(3,9)
# the event times are capture times.
# Righ-censoring time C~U(0,c).
# Assume the population size n is known.
# The seed of the simulation is 2000.
# We considered the sample size n=50, 200, 500.
# We considered the censoring proprotion to be 20%, 30% and 40%.
# For apporoximating the non-linear component we used cubic spline.
# We selected the #of basis knots/basis function using BIC. we considered 4,5,6,7,8 basis functions to select the 
# one that minimizes the BIC. We did it for each of the 1000 runs/simulations.

sink("Example1 Results seed 2000.txt", append=FALSE)                           #Right-censoring, no truncation;
cat("The program started: ", date(), "\n \n")
cat("Run R code Example1.R to get the results:", "\n")

rm(list=ls())
library(ahaz)
library(fda)
library(survival)
library(MASS)

options(width=60,length=200)
seed<-2000
cat("Random seed used in simulation:", seed, "\n")
set.seed(seed)

#Step 1: Data Generating Process
survival.data<-function(n,cv,ttime, beta0,en) #en=effective sample size;
{     blength=length(beta0)
z1<-runif(n,min=1,max=4)-2.5         # E(z1)=0, covariate for a linear component;
z2<-rbinom(n,size=1,p=0.5)-0.5       # E(z2)=0 covariate for a linear component;
W1<-runif(n,min=Wa,max=Wb)           # covariate for the nonlinear components;
W2<-runif(n,min=Wa,max=Wb)           # covariate for the nonlinear components;
gw1<-sin(pi*(W1/3-1))                 # 1st nonlinear component;
gw2<-0.3*((W2-6)^2-3)                # 2nd nonlinear component;
gw1<-gw1-mean(gw1)                   # such that mean of gw1 is zero for identifiability;
gw2<-gw2-mean(gw2)                   # such that mean of gw2 is zero for identifiability;

srate<-5+beta0[1]*z1+beta0[2]*z2+gw1+gw2    # harazd rate;
stime<-rexp(n,rate=srate)                   # survival time;
ltime<-c(rep(ttime,n))                      #truncation time;
rtime<-stime-ltime                          #residual time

##Generate enough survival times first, then get specified sample size;
rstime<-subset(stime, (rtime>0))
rltime<-subset(ltime, (rtime>0))
rrtime<-subset(rtime, (rtime>0))
rz1<-subset(z1, (rtime>0))
rz2<-subset(z2, (rtime>0))
rW1<-subset(W1, (rtime>0))
rW2<-subset(W2, (rtime>0))
sn=length(rstime)
if (sn<en) {
  cat("c(sn, en)=", c(sn,en), "\n")    
  cat("This generated data set does not have enough samples, move to the next", "\n \n")
  break
}
ind1<-sample(1:sn, en, replace=FALSE)
z1<-rz1[ind1]
z2<-rz2[ind1]
W1<-rW1[ind1]
W2<-rW2[ind1]
ostime<-rstime[ind1]  #observed stime;
oltime<-rltime[ind1]  #observed ltime;
ortime<-rrtime[ind1]
ctime1<-runif(en)*cv  #censoring time;
ctime<-oltime+ctime1 
delta<-1*(ostime<=ctime)
otime<-delta*ostime+(1-delta)*ctime
sdata<-data.frame(otime=otime, delta=delta, lt=oltime, z1=z1,z2=z2,W1=W1,W2=W2);
sdata 
}


#Step 2: Estimate parameters and splines;

ParaEst<-function(w,dfB.W)               # w is the generated dataset, in this case 'survival.data'.
{      
  Wnknots=dfB.W+2-border      #Using dfB.W as arguments instead of Wnknots since dfB.W is used in BIC 
  #dfB.W=border+Wnknots-2     #number of B-spline basis function in estimating g(w)
  otime <- w[,1]                     #observation time
  delta <- w[,2]                     #censoring indicator
  lt<-w[,3]                          #left trancation time
  Z<- as.matrix(w[,4:5])
  W1 <- w[,6]
  W2 <- w[,7]
  
  ###Estimation and prediction of nonlinear functions ######
  ###Estimation of g(W1) ######
  
  knotspos_W1<-rep(0,Wnknots)
  # for(i in 1:Wnknots) {knotspos_W1[i]<-range(W1)[1]+(i-1)*stepW1}
  W1.quant=quantile(W1, probs=seq(0,1,by=1/(Wnknots-1)))
  for(i in 1:Wnknots) {knotspos_W1[i]<-W1.quant[i]}               #Use quantiles;
  
  knotspos_W1[1]=Wa-Wadj       # knotspos_W1[1]             #-0.0001, use a slightly wider interval so that 
  # prediction can be performed;
  knotspos_W1[Wnknots]=Wb+Wadj # knotspos_W1[Wnknots]       #+0.0001
  
  gbasis1<- bsplineS(W1, breaks=c(knotspos_W1), norder=dfB.W-(Wnknots-2), nderiv=0)  # for Wnknot<3, there is no interior knot
  #Use library(fda), normalized B-splines, dfB is the number of B-splines used for g();
  gbasis1<-gbasis1[,-dfB.W]
  mean.gbasis1<-matrix(apply(gbasis1,2,mean),1,dfB.W-1) 
  # print(apply(gbasis,1,sum))         #sum is 1, the basis functions are linear dependent, so we use 1 less df in
  #backfitting linear models;
  Ng1=nrow(gbasis1)
  mean.mat1=kronecker(mean.gbasis1, matrix(1,Ng1,1))
  gbasis1=gbasis1-mean.mat1   #Ensures E(g(w))=0; 
  
  ###Prediction of g(W1) ######
  
  W.grid<-seq(Wa+Wadj,Wb-Wadj, (Wb-Wa-2*Wadj)/(ngrid-1))  #Divide [Wa,Wb] into (ngrid-1) equally spaced intervals;
  pred.gbasis1<-bsplineS(W.grid, breaks=c(knotspos_W1), norder=dfB.W-(Wnknots-2), nderiv=0) 
  pred.gbasis1<-pred.gbasis1[,-dfB.W]
  mean.gpbasis1<-matrix(apply(pred.gbasis1,2,mean),1,dfB.W-1)   
  Ngp1=nrow(pred.gbasis1)
  mean.gpmat1=kronecker(mean.gpbasis1, matrix(1,Ngp1,1))
  pred.gbasis1=pred.gbasis1-mean.gpmat1   #Ensures E(g(w))=0;
  
  
  ###Estimation of g(W2) ######
  
  knotspos_W2<-rep(0,Wnknots)
  W2.quant=quantile(W2, probs=seq(0,1,by=1/(Wnknots-1)))
  for(i in 1:Wnknots) {knotspos_W2[i]<-W2.quant[i]}               #Use quantiles;
  
  knotspos_W2[1]=Wa-Wadj       # knotspos_W1[1]             #-0.0001, use a slightly wider interval so that 
  # prediction can be performed;
  knotspos_W2[Wnknots]=Wb+Wadj # knotspos_W1[Wnknots]       #+0.0001
  gbasis2<- bsplineS(W2, breaks=c(knotspos_W2), norder=dfB.W-(Wnknots-2), nderiv=0)  # for Wnknot<3, there is no interior knot
  #Use library(fda), normalized B-splines, dfB is the number of B-splines used for g();
  gbasis2<-gbasis2[,-dfB.W]
  mean.gbasis2<-matrix(apply(gbasis2,2,mean),1,dfB.W-1) 
  Ng2=nrow(gbasis2)
  mean.mat2=kronecker(mean.gbasis2, matrix(1,Ng2,1))
  gbasis2=gbasis2-mean.mat2   #Ensure E(g(w))=0; 
  
  ###Prediction of g(W2) ######
  
  W.grid<-seq(Wa+Wadj,Wb-Wadj, (Wb-Wa-2*Wadj)/(ngrid-1))  #Divide [Wa,Wb] into (ngrid-1) equally spaced intervals;
  pred.gbasis2<-bsplineS(W.grid, breaks=c(knotspos_W2), norder=dfB.W-(Wnknots-2), nderiv=0) 
  # pred.gbasis<-predict(gbasis,W.grid)
  pred.gbasis2<-pred.gbasis2[,-dfB.W]
  mean.gpbasis2<-matrix(apply(pred.gbasis2,2,mean),1,dfB.W-1)   
  Ngp2=nrow(pred.gbasis2)
  mean.gpmat2=kronecker(mean.gpbasis2, matrix(1,Ngp2,1))
  pred.gbasis2=pred.gbasis2-mean.gpmat2   #Ensure E(g(w))=0; 
  
  crate<-1-mean(delta)                 #right censoring rate;
  
  ####Estimation of the PLAHM########
  
  surv <- Surv(lt,otime,delta)           #lt=left trancation time, otime=observed time, delta=censoring indicator;
  Z<-cbind(Z,gbasis1,gbasis2)
  ahaz<-ahaz(surv,Z)
  out<-summary(ahaz)
  est<-out$coefficients[,1]
  est_se<-out$coefficients[,2]
  var<-out$cov
  
  ###Estimates
  
  beta<-est[1:bp]                         #beta estimates;
  gamma1<-est[(bp+1):(bp+dfB.W-1)]        #b-spline coefficient estimates for g(W1);
  gamma2<-est[(bp+dfB.W):(2*dfB.W)]       #b-spline coefficient estimates for g(W2);
  beta_se<-est_se[1:bp]                   #SE of beta
  gamma1_se<-est_se[(bp+1):(bp+dfB.W-1)]  #sE of b-spline coefficient of g(W1);
  gamma2_se<-est_se[(bp+dfB.W):(2*dfB.W)] #sE of b-spline coefficient of g(W2);
  gest1<-pred.gbasis1%*%gamma1            #Use B-Spline to predict g(W1) at ngrid points;
  gest2<-pred.gbasis2%*%gamma2            #Use B-Spline to predict g(W2) at ngrid points;
  beta_var<-var[1:bp,1:bp]                #variance-covariance matrix of beta
  gamma1_var<-var[(bp+1):(bp+dfB.W-1),(bp+1):(bp+dfB.W-1)]   #variance-covariance matrix of of the B-spline coefficients of g(W1);
  gamma2_var<-var[(bp+dfB.W):(2*dfB.W),(bp+dfB.W):(2*dfB.W)] #variance-covariance matrix of of the B-spline coefficients of g(W1);
  
  ###Coverage probability
  
  LL<-beta-(1.96*beta_se)                          #Lower limit of the CI for each beta;
  UL<-beta+(1.96*beta_se)                          #Upper limit of the CI for each beta;
  cov_ind<-1*(beta0>=LL & beta0<=UL)               #Coverage probability for each beta;
  
  invC<-solve(beta_var)
  Waldstat<- t(beta-beta0)%*%invC%*%(beta-beta0);
  cov_joint<-1*(t(beta-beta0)%*%invC%*%(beta-beta0)<=qchisq(.95, 2)) #Joint coverage probabity of beta's;
  
  ##based on SE(gammas')
  
  V_gest1<-matrix(0,ngrid,1)
  V_gest2<-matrix(0,ngrid,1)
  for (l in 1:ngrid){
    V_gest1[l,]<-t(pred.gbasis1[l,])%*%gamma1_var%*%pred.gbasis1[l,]
    V_gest2[l,]<-t(pred.gbasis2[l,])%*%gamma2_var%*%pred.gbasis2[l,]}
  SE_gest1<-sqrt(V_gest1)
  SE_gest2<-sqrt(V_gest2)
  LL_gest1<-gest1-(1.96*SE_gest1)                  #Pointwise confidence interval for each nonlinear function;
  UL_gest1<-gest1+(1.96*SE_gest1)                  #based on SE of the B-spline coefficients
  LL_gest2<-gest2-(1.96*SE_gest2)
  UL_gest2<-gest2+(1.96*SE_gest2)
  
  #### BIC to select the number of optimul knots
  
  D<-ahaz$D
  d<-ahaz$d
  B<-ahaz$B
  
  m<-(t(d)%*%solve(B)%*%d)/(t(d)%*%solve(D)%*%d)   # Following Rasmussen and Scheike (JSS, 2012)
  LF<-(t(est)%*%D%*%est)-(2*t(est)%*%d)            # Loss function
  BIC<-(m*LF)+((bp+2*(dfB.W-1))*log(en))               # BIC function
  
  
  list(beta, beta_se, cov_ind, cov_joint, gest1, gest2, LL_gest1, LL_gest2, UL_gest1, UL_gest2, W.grid, BIC, crate,dfB.W)
  
}

#########################################
#CHANGE VALUES OF PARAMETERS HERE
######################################### 

list1<-NULL   #Define two lists to hold values for ploting figures of the two nonlinear functions;
list2<-NULL

j.vec=c(50,200,500)
rcv.vec=c(1.05, 0.67, 0.46) #For beta.true=(0.3, 0.5), cp=20, 30, 42%;

for (j in 1:3){    #j=50, 200 and 500 are sampl sizes
  for (i in 1:3){    #rcv= 1.05, .67, .46, 
    
    if (j==1) en=j.vec[1] else if (j==2) en=j.vec[2] else en=j.vec[3]
    if (i==1) rcv=rcv.vec[1] else if (i==2) rcv=rcv.vec[2] else rcv=rcv.vec[3]
    
    cat("Sample size and rcv value=", c(j.vec[j], rcv.vec[i]), "\n \n")
    
    n<-99999              # non-truncated Sample size
    B<-1000               # Simulation times.
    beta0<-c(0.3, 0.5)    # True Parameter values.
    bp<-length(beta0)     # Number of variables in the linear part
    Wa=3                  # Interval [Za,Zb] for the covariate z3;
    Wb=9 
    Wadj=0.00             # Adjustment of the boundary knots in B-Splines, to remove boundary effects;
    
    border=4 
    ltr=0.0;              #left truncation 
    startdf<-4
    ngrid<-en
    
    Num.Basis<-seq(4,6,by=1)
    NB=length(Num.Basis) 
    
    beta.Est<- array(0, c(B, bp, NB))
    beta_se.Est <- array(0, c(B, bp, NB))
    beta_cov_ind.Est <- array(0, c(B, bp, NB))
    beta_cov_joint.Est <- array(0, c(B, 1, NB))
    
    g1.Est<-array(0, c(B, ngrid, NB)) 
    LL_gest1<- array(0, c(B,ngrid, NB))
    UL_gest1<- array(0, c(B,ngrid, NB))
    g2.Est<-array(0, c(B, ngrid, NB)) 
    LL_gest2<- array(0, c(B,ngrid, NB))
    UL_gest2<- array(0, c(B,ngrid, NB))
    
    W.Grid<-rep(0,ngrid) 
    
    BIC.funV <- matrix(0, B, NB)                       #Value of BIC function;
    crate <- matrix(0, B, NB)                               #Censoring rate;
    dfB.WV <- matrix(0, B, NB)                         #Value of number of B-spline basis functions for g(w)'s;
    
    ### Final results selected by BIC
    
    beta.Est.BIC<- matrix(0, B, bp)
    beta_se.Est.BIC <- matrix(0, B, bp)
    beta_cov_ind.Est.BIC <-matrix(0, B, bp)
    beta_cov_joint.Est.BIC <-matrix(0, B, 1)
    
    g1.Est.BIC<-matrix(0, B, ngrid)
    LL_gest1.BIC<-matrix(0,B,ngrid) 
    UL_gest1.BIC<-matrix(0,B,ngrid) 
    g2.Est.BIC<-matrix(0, B, ngrid)
    LL_gest2.BIC<-matrix(0,B,ngrid) 
    UL_gest2.BIC<-matrix(0,B,ngrid) 
    
    crate.BIC <- matrix(0, B, 1)                          #Censoring rate;
    BIC.funV.min <- matrix(0, B, 1)                       #Value of BIC function;
    dfB.WV.BIC <- matrix(0, B, 1)                         #Value of number of B-spline basis functions for g(w)'s;
    
    
    for (k in 1:B){
      #cat("\n", "This is the",  k , "th simulation", "\n")
      
      ngrid<-en
      newdata<-survival.data(n,rcv, ltr, beta0,en)
      
      for (s in 1:NB){   #loop for different number of basis functions;
        dfB.W=Num.Basis[s]
        # cat("\n", "This is the",  s , "th number of basis functions", ",", "\n \n")
        out<-ParaEst(w=newdata, dfB.W=dfB.W) 
        #list(beta, beta_se, cov_ind, cov_joint, gest1, LL_gest1, UL_gest1, W.grid, BIC, crate,dfB.W)
        beta.Est[k,,s]=out[[1]]
        beta_se.Est[k,,s]=out[[2]]
        beta_cov_ind.Est[k,,s]=out[[3]]
        beta_cov_joint.Est[k,,s]=out[[4]]
        g1.Est[k,,s]=out[[5]]
        g2.Est[k,,s]=out[[6]]
        LL_gest1[k,,s]=out[[7]]
        LL_gest2[k,,s]=out[[8]]
        UL_gest1[k,,s]=out[[9]]
        UL_gest2[k,,s]=out[[10]]
        W.Grid=out[[11]]
        BIC.funV[k,s]=out[[12]]
        crate[k,s]=out[[13]]
        dfB.WV[k,s]=out[[14]]
      } #End of s loop of different number basis functions;
      
      ### Summarize BIC results for each ith simulation;
      
      dfB.temp<-which.min(BIC.funV[k,])            #Value of min BIC function;
      dfB.WV.BIC[k,]<-dfB.WV[i, dfB.temp]          #Selected value of number of B-spline basis functions for g(w)'s;
      BIC.funV.min[k,]<-BIC.funV[k, dfB.temp]      #If use min(BIC.funV[i,]), it might be NaN values;
      
      beta.Est.BIC[k,]<-beta.Est[k,,dfB.temp] 
      beta_se.Est.BIC[k,]<- beta_se.Est[k,,dfB.temp]
      beta_cov_ind.Est.BIC[k,] <-beta_cov_ind.Est[k,,dfB.temp] 
      beta_cov_joint.Est.BIC[k,] <-beta_cov_joint.Est[k,,dfB.temp] 
      
      crate.BIC[k,] <- crate[k,dfB.temp]                                #Censoring rate;
      
      g1.Est.BIC[k,]<-g1.Est[k,,dfB.temp]  
      LL_gest1.BIC[k,]<-LL_gest1[k,,dfB.temp]
      UL_gest1.BIC[k,]<-UL_gest1[k,,dfB.temp]
      g2.Est.BIC[k,]<-g2.Est[k,,dfB.temp]  
      LL_gest2.BIC[k,]<-LL_gest2[k,,dfB.temp]
      UL_gest2.BIC[k,]<-UL_gest2[k,,dfB.temp]
    } #end(s)-main loop
    
    ###Estimation precision of the linear part####
    
    #hist(beta.Est.BIC[,2])
    beta.ave<-colMeans(beta.Est.BIC)
    AESE<-colMeans(beta_se.Est.BIC)
    SSE<-apply(beta.Est.BIC, 2, sd)  #empirical se of the estimates
    beta.bias<-beta.ave-beta0
    
    cov_ind<-colMeans(beta_cov_ind.Est.BIC)
    cov_joint<-colMeans(beta_cov_joint.Est.BIC)
    crate<-colMeans(crate.BIC)
    ave_basis<-colMeans(dfB.WV.BIC)
    
    
    cat("The sample size for this simulation is n=   ", en, "\n",
        "with censoring proportion:                  ", crate, "\n",
        "Simulation times:                           ", B,"\n",
        "True beta  value:                           ", round(beta0,3), "\n",
        "Averaged beta estimate:                     ", round(beta.ave,3), "\n",
        "Estimated beta bias:                        ", round(beta.bias,3), "\n",
        "AESE of beta:                               ", round(AESE,3), "\n",
        "SSE of beta:                                ", round(SSE,3), "\n",
        "Coverage Probability of betas:              ", round(cov_ind,3), "\n",
        "Normal 95% coverage probability;            ", round(cov_joint,3), "\n",
        "Averaged # of basis function used;          ", round(ave_basis,3), "\n \n")
    
    ###estimation precision of the nonlinear function
    
    estimated.ave.function1<-colMeans(g1.Est.BIC)
    LL1<-colMeans(LL_gest1.BIC)  # based on AESE
    UL1<-colMeans(UL_gest1.BIC)  # based on AESE
    W.Grid<-W.Grid
    true.nonpar1<-sin(pi*(W.Grid/3-1)) 
    g1.quant=matrix(0, 2, ngrid)
    for (i in 1:ngrid){
      g1.quant[,i]=quantile(g1.Est.BIC[,i], probs=c(0.025, 0.975), na.rm=TRUE)
    }
    
    estimated.ave.function2<-colMeans(g2.Est.BIC)
    LL2<-colMeans(LL_gest2.BIC)  # based on AESE
    UL2<-colMeans(UL_gest2.BIC)  # based on AESE
    W.Grid<-W.Grid
    true.nonpar2<-0.3*((W.Grid-6)^2-3) 
    g2.quant=matrix(0, 2, ngrid)
    for (i in 1:ngrid){
      g2.quant[,i]=quantile(g2.Est.BIC[,i], probs=c(0.025, 0.975), na.rm=TRUE)
    }
    
    
    fun1<-list(W.Grid, true.nonpar1, estimated.ave.function1, LL1, UL1, g1.quant[1,], g1.quant[2,])
    list1[[length(list1)+1]]<-fun1
    
    fun2<-list(W.Grid, true.nonpar2, estimated.ave.function2, LL2, UL2, g2.quant[1,], g2.quant[2,])
    list2[[length(list2)+1]]<-fun2
  }  #End of i loop;
}  #End of j loop;  

##Plot graphs;

dev.new()
pdf("Example 1 function 1-seed2000.pdf")
par(mfrow=c(3,3))
#par(mfcol=c(3,3))  #Plot by column;

cr1=c(20, 30, 40)
cr=rep(cr1,3)
en=c(rep(50,3), rep(200, 3), rep(500,3))

for (k in 1:9){
  W.Grid=c(list1[[k]][[1]])
  true.nonpar1=c(list1[[k]][[2]])
  estimated.ave.function1=c(list1[[k]][[3]])
  LL1=c(list1[[k]][[4]])
  UL1=c(list1[[k]][[5]])
  g1.quant1=c(list1[[k]][[6]])
  g1.quant2=c(list1[[k]][[7]])
  
  plot(W.Grid,true.nonpar1,lwd=2, type="l", lty=1, xlab="W1", ylab=expression(paste(varphi[1],"(W1)")), ylim=c(-6,6), main=paste("n= ", en[k], ", cp= ", cr[k],"%"))
  #plot(W.Grid,true.nonpar1,lwd=2, type="l", lty=1, xlab="W1", ylab=expression(paste(varphi[1],"(W1)")), ylim=c(min(LL1),max(UL1)), main=paste("n= ", en[k], ", cp= ", cr[k],"%"))
  
  lines(W.Grid,estimated.ave.function1,lwd=2, type="l", lty=2)
  lines(W.Grid,LL1,lwd=2, type="l", lty=3)
  lines(W.Grid,UL1,lwd=2, type="l", lty=3)
  lines(W.Grid,g1.quant1,lwd=2, type="l", lty=4)
  lines(W.Grid,g1.quant2,lwd=2, type="l", lty=4)
}  #End of k loop;
dev.off()

dev.new()
pdf("Example 1 function 2-seed2000.pdf")
par(mfrow=c(3,3))
#par(mfcol=c(3,3))  #Plot by column;

cr1=c(20, 30, 40)
cr=rep(cr1,3)
en=c(rep(50,3), rep(200, 3), rep(500,3))

for (k in 1:9){
  W.Grid=c(list2[[k]][[1]])
  true.nonpar2=c(list2[[k]][[2]])
  estimated.ave.function2=c(list2[[k]][[3]])
  LL2=c(list2[[k]][[4]])
  UL2=c(list2[[k]][[5]])
  g2.quant1=c(list2[[k]][[6]])
  g2.quant2=c(list2[[k]][[7]])
  
  plot(W.Grid,true.nonpar2,lwd=2, type="l",xlab="W2", ylab=expression(paste(varphi[2],"(W2)")), ylim=c(-5,10), main=paste("n= ", en[k], ", cp= ", cr[k],"%"))
  #plot(W.Grid,true.nonpar2,lwd=2, type="l",xlab="W2", ylab=expression(paste(varphi[2],"(W2)")), ylim=c(min(LL2),max(UL2)), main=paste("n= ", en[k], ", cp= ", cr[k],"%"))
  lines(W.Grid,estimated.ave.function2,lwd=2, type="l", lty=2)
  lines(W.Grid,LL2,lwd=2, type="l", lty=3)
  lines(W.Grid,UL2,lwd=2, type="l", lty=3)
  lines(W.Grid,g2.quant1,lwd=2, type="l", lty=4)
  lines(W.Grid,g2.quant2,lwd=2, type="l", lty=4)
}  #End of k loop;
dev.off()

cat("The program ended:", date(), "\n")
sink()
