######################################################################################
#  Supplementary Materils.   Dec. 31, 2016                           																	         #
#  R code for Example 4 in Afzal et al. (2016) for the journal of Statisical Modelling titled                         #
# "Estimation of Partly Linear Additive Hazards Model with Left Truncated and Right Censored Data";    #
######################################################################################

#  Reproducing Yin et al. (2008) Simulation Table 1 using our B-spline based method as referees recommended;
#  We selected the #of basis knots/basis function using BIC. we considered 4,5,6,7,8 basis functions to select the 
#  one that minimizes the BIC. We did it for each of the 500 run/simulations.

sink("Reproducing Yin etal. (2008) Table 1 Using Bspline Approach-cr=28%-n=500.txt", append=FALSE)   
      #Right-censoring, no truncation; 
      #The Yin et al. (2008) indicted that the censoring rate is 25%, our simulation study shows it is actually 28%;

cat("The program started: ", date(), "\n \n")
rm(list=ls())
library(ahaz)            # for additive hazards model;
library(fda)              # for generating basis functions;
library(survival)
library(MASS)

options(width=60,length=200)
seed<-2016
set.seed(seed)

#Step 1: Data Generating Process
survival.data<-function(n,ttime, gamma0,lambda_0, tau, en){    #en=effective sample size;
W<-runif(n,min=Wa,max=Wb)
bw.func<-1.2 + sin(2*W)
Z<-runif(n,min=0,max=1)              
V<-rbinom(n,1,prob=0.5)             # covariate for the linear part;
aw<-0.2*W                                   # nonlinear component;
#aw<-aw-mean(aw)                      # such that mean of aw1 is zero for identifiability; S
                                                      #  since we are interested in derivative of a(w), center is not needed here;

srate<-lambda_0+bw.func*Z+gamma0*V+aw       # harazd rate;
stime<-rexp(n,rate=srate)                   # survival time;
ltime<-c(rep(ttime,n))                         # truncation time; then ttime==0, produce not truncation;
rtime<-stime-ltime                              # residual time

##Generate enough survival times first, then get specified sample size;
rstime<-subset(stime, (rtime>0))
rltime<-subset(ltime, (rtime>0))
rrtime<-subset(rtime, (rtime>0))
rW<-subset(W, (rtime>0))
rZ<-subset(Z, (rtime>0))
rV<-subset(V, (rtime>0))
sn=length(rstime)
if (sn<en) {
  cat("c(sn, en)=", c(sn,en), "\n")    
  cat("This generated data set does not have enough samples, move to the next", "\n \n")
  next
}
ind1<-sample(1:sn, en, replace=FALSE)
W<-rW[ind1]
Z<-rZ[ind1]
V<-rV[ind1]
ostime<-rstime[ind1]  #observed stime;
oltime<-rltime[ind1]  #observed ltime;
ortime<-rrtime[ind1]
ctime1<-runif(en, tau/2,(3*tau)/2)   #censoring time; 
ctime2<-pmin(tau, ctime1)             
#ctime<-oltime+ctime1                   #Using ctime1 as censoring rate gives approximately 25% censoring; 
ctime<-oltime+ctime2                     #Using ctime2 increase the censoring rate to almost 28%;
delta<-1*(ostime<=ctime)
otime<-delta*ostime+(1-delta)*ctime
sdata<-data.frame(otime=otime, delta=delta, W=W, Z=Z, V=V);
sdata 
}

### Generating basis functions, and derivative of basis functions evaluated at W for Estimation;

Bsplines<-function(W,dfB.W){
  Wnknots=dfB.W+2-border      #Using dfB.W as arguments instead of Wnknots since dfB.W is used in BIC 
  #gbasis<-matrix(NA, en, dfB.W-1)              
  knotspos_W<-rep(0,Wnknots)                        ## Knots position;              
  W.quant=quantile(W, probs=seq(0,1,by=1/(Wnknots-1)))
  for(i in 1:Wnknots) {knotspos_W[i]<-W.quant[i]}
  knotspos_W[1]=Wa-Wadj       # knotspos_W1[1]            
  knotspos_W[Wnknots]=Wb+Wadj # knotspos_W1[Wnknots] ## Use quantiles for knot positions;
  gbasis.all<- bsplineS(W, breaks=c(knotspos_W), norder=dfB.W-(Wnknots-2), nderiv=0)  ## For Wnknot<3, there is no interior knot;
  gbasis<-gbasis.all[,-dfB.W]
  # mean.gbasis<-matrix(apply(gbasis,2,mean),1,dfB.W-1)  
  # Ng=nrow(gbasis)
  # mean.mat=kronecker(mean.gbasis, matrix(1,Ng,1))
  # gbasis=gbasis-mean.mat   #Ensure E(g(w))=0;
  
  #### Generating basis functions for Prediction;
  #pred.gbasis<-matrix(NA, en, dfB.W-1)
  W.Grid<-seq(0.5, 2.5, 0.5)       
  pred.gbasis.all<-bsplineS(W.Grid, breaks=c(knotspos_W), norder=dfB.W-(Wnknots-2), nderiv=0) 
  #pred.gbasis<-pred.gbasis.all[,-dfB.W]
  # mean.gpbasis<-matrix(apply(pred.gbasis,2,mean),1,dfB.W-1)   
  # Ngp=nrow(pred.gbasis)
  # mean.gpmat=kronecker(mean.gpbasis, matrix(1,Ngp,1))
  # pred.gbasis=pred.gbasis-mean.gpmat                           ## Ensure E(g(w))=0;  
  
  #### Generating basis functions for Prediction for the first derivate;
  deriv.pred.gbasis<-matrix(NA, en, dfB.W-1)
  deriv.pred.gbasis.all<-bsplineS(W.Grid, breaks=c(knotspos_W), norder=dfB.W-(Wnknots-2), nderiv=1) 
  deriv.pred.gbasis<-deriv.pred.gbasis.all[,-dfB.W]
  # deriv.mean.gpbasis<-matrix(apply(deriv.pred.gbasis,2,mean),1,dfB.W-1)   
  # Ngp=nrow(deriv.pred.gbasis)
  # deriv.mean.gpmat=kronecker(deriv.mean.gpbasis, matrix(1,Ngp,1))
  # deriv.pred.gbasis=deriv.pred.gbasis-deriv.mean.gpmat  
  
  list(gbasis.all, gbasis, pred.gbasis.all, deriv.pred.gbasis, W.Grid)   #gbasis.all for bw, gbasis for aw;       
}

Estimation<-function(data){
  otime <- data[,1]                     #observation time
  delta <- data[,2]                      #censoring indicator
  W <-data[,3]                    
  Z <- data[,4]                    
  V<-data[,5] 
  basis<-Bsplines(W,dfB.W)
  bw.basis<-basis[[1]]
  aw.basis<-basis[[2]]
  bw.pred.basis<-basis[[3]]
  aw.deriv.pred.basis<-basis[[4]]
  W.Grid<-basis[[5]]
  b.basis<-bw.basis*Z                  #covariate created for bw;
  
  crate<-1-mean(delta)
  
  ####Estimation in AHAZ
  
  surv <- Surv(otime,delta)           #otime=observed time, delta=censoring indicator;
  cov<-cbind(V, b.basis, aw.basis)
  ahaz<-ahaz(surv,cov)
  out<-summary(ahaz)
  est<-out$coefficients[,1]
  est_se<-out$coefficients[,2]
  var<-out$cov
  
  #################
  ### For GAMMA ###
  #######################################################
  gamma_est<-est[1:bp]        # estimate of gamma0;
  gamma_se<-est_se[1:bp]      # SE of the estimate
  
  ###Coverage probability
  
  LL<-gamma_est-(1.96*gamma_se)                          #Lower limit of the CI for each beta;
  UL<-gamma_est+(1.96*gamma_se)                          #Upper limit of the CI for each beta;
  cov_prob.gamma<-1*(gamma0>=LL & gamma0<=UL)            #Coverage probability for each beta;
  
  #######################################################
  
  ################
  ### For B(W) ###
  #######################################################
  bw.coef<-est[(bp+1):(bp+dfB.W)]
  bw.coef.se<-est_se[(bp+1):(bp+dfB.W)]      #sE of b-spline coefficient of g(W1);
  bw.est<-bw.pred.basis%*%bw.coef            #Use B-Spline to predict g(W1) at ngrid points;
  bw.coef.var<-var[(bp+1):(bp+dfB.W),(bp+1):(bp+dfB.W)]   #variance-covariance matrix of of the B-spline coefficients of B(W);
  
  ### performance for B(W); try for a single grid point 0.5 first;
  
  Var.bw.est<-matrix(0,length(W.Grid),1)
  SE.bw.est<-matrix(0,length(W.Grid),1)
  LL.bw.est<-matrix(0,length(W.Grid),1)
  UL.bw.est<-matrix(0,length(W.Grid),1)
  cov_prob.bw<-matrix(0,length(W.Grid),1)
  true.bw<-matrix(NA, length(W.Grid), 1)
  for(i in 1: length(W.Grid)){
    true.bw[i,]<-1.2+sin(2*W.Grid[i])
  }
  for (l in 1:length(W.Grid)){
    Var.bw.est[l,]<-t(bw.pred.basis[l,])%*%bw.coef.var%*%bw.pred.basis[l,]
    SE.bw.est[l,]<-sqrt(Var.bw.est[l,])
    LL.bw.est[l,]<-bw.est[l,]-(1.96*SE.bw.est[l,])                  #Pointwise confidence interval for each nonlinear function;
    UL.bw.est[l,]<-bw.est[l,]+(1.96*SE.bw.est[l,])                  #based on SE of the B-spline coefficients
    cov_prob.bw[l,]<-1*(true.bw[l]>=LL.bw.est[l,] & true.bw[l]<=UL.bw.est[l,])
  }
  ########################################################
  
  ########################################################
  ### For derivate of Alpha ####
  ########################################################
  aw.coef<-est[(bp+dfB.W+1):(2*dfB.W)] 
  aw.coef.se<-est_se[(bp+dfB.W+1):(2*dfB.W)]
  deriv.aw.est<-aw.deriv.pred.basis%*%aw.coef  
  aw.coef.var<-var[(bp+dfB.W+1):(2*dfB.W),(bp+dfB.W+1):(2*dfB.W)] #variance-covariance matrix of of the B-spline coefficients of g(W);
  
  Var.deriv.aw.est<-matrix(0,length(W.Grid),1)
  SE.deriv.aw.est<-matrix(0,length(W.Grid),1)
  LL.deriv.aw.est<-matrix(0,length(W.Grid),1)
  UL.deriv.aw.est<-matrix(0,length(W.Grid),1)
  cov_prob.deriv.aw<-matrix(0,length(W.Grid),1)
  true.deriv.aw<-0.2
  for (l in 1:length(W.Grid)){
    Var.deriv.aw.est[l,]<-t(aw.deriv.pred.basis[l,])%*%aw.coef.var%*%aw.deriv.pred.basis[l,]
    SE.deriv.aw.est[l,]<-sqrt(Var.deriv.aw.est[l,])
    LL.deriv.aw.est[l,]<-deriv.aw.est[l,]-(1.96*SE.deriv.aw.est[l,])                  #Pointwise confidence interval for each nonlinear function;
    UL.deriv.aw.est[l,]<-deriv.aw.est[l,]+(1.96*SE.deriv.aw.est[l,])                  #based on SE of the B-spline coefficients
    cov_prob.deriv.aw[l,]<-1*(true.deriv.aw>=LL.deriv.aw.est[l,] & true.deriv.aw<=UL.deriv.aw.est[l,])
  }
  ############################################################################
  
  #############################################
  #### BIC to select the number of optimul knots
  #############################################
  
  D<-ahaz$D
  d<-ahaz$d
  B<-ahaz$B
  
  m<-(t(d)%*%solve(B)%*%d)/(t(d)%*%solve(D)%*%d)   # Following Rasmussen and Scheike (JSS, 2012)
  LF<-(t(est)%*%D%*%est)-(2*t(est)%*%d)            # Loss function
  BIC<-(m*LF)+(length(est)*log(en))                # BIC function
  
  ##############################################
  list(gamma_est,gamma_se, cov_prob.gamma, bw.est, SE.bw.est, cov_prob.bw, deriv.aw.est, SE.deriv.aw.est, cov_prob.deriv.aw, crate, BIC)
  #########################################################
} # end of estimation

#########################################
#CHANGE VALUES OF PARAMETERS HERE
######################################### 

n<-99999              # non-truncated Sample size
B<-500                # Simulation times.
gamma0<-1             # True Parameter values.
lambda_0<-0.5         # baseline hazard;
ttime=0               # truncation time;
Wa=0                  # Interval [Wa,Wb];
Wb=3 
Wadj=0.00             # Adjustment of the boundary knots in B-Splines, to remove boundary effects;
tau=0.86
border=4 
startdf<-4
Num.Basis<-seq(4,8,by=1)
NB=length(Num.Basis) 
bp<-length(gamma0)
W.Grid<-seq(0.5, 2.5, 0.5)
true.bw<-matrix(NA, length(W.Grid), 1)
for(i in 1: length(W.Grid)){
  true.bw[i,]<-1.2+sin(2*W.Grid[i])
}
true_deriv_aw<-0.2
j.vec=c(200,400)

for (j in 1:2){    #j=200 and 400 are sampl sizes
cat("Sample size=", j.vec[j], "\n \n")
if (j==1) en=j.vec[1] else en=j.vec[2]

gamma_est<-array(NA, c(1, B, NB))
gamma_se<-array(NA, c(1, B, NB))
cov_prob.gamma<-array(NA, c(1, B, NB))
bw.est<-array(NA, c(length(W.Grid), B, NB))
SE.bw.est<-array(NA, c(length(W.Grid), B, NB))
cov_prob.bw<-array(NA, c(length(W.Grid), B, NB))
deriv.aw.est<-array(NA, c(length(W.Grid), B, NB))
SE.deriv.aw.est<-array(NA, c(length(W.Grid), B, NB))
cov_prob.deriv.aw<-array(NA, c(length(W.Grid), B, NB))
BIC.funV <- matrix(NA, B, NB)                       #Value of BIC function;
crate <- matrix(NA, B, NB)                          #Censoring rate;
dfB.WV <- matrix(NA, B, NB)

gamma_est.BIC<-matrix(NA, 1, B)
gamma_se.BIC<-matrix(NA, 1, B)
cov_prob.gamma.BIC<-matrix(NA,1, B)
bw.est.BIC<-matrix(NA, length(W.Grid), B)
SE.bw.est.BIC<-matrix(NA, length(W.Grid), B)
cov_prob.bw.BIC<-matrix(NA, length(W.Grid), B)
deriv.aw.est.BIC<-matrix(NA, length(W.Grid), B)
SE.deriv.aw.est.BIC<-matrix(NA, length(W.Grid), B)
cov_prob.deriv.aw.BIC<-matrix(NA, length(W.Grid), B)
BIC.funV.min <- matrix(NA, 1, B)                       #Value of BIC function;
crate.BIC <- matrix(NA, 1, B)                          #Censoring rate;
dfB.WV.BIC <- matrix(NA, 1, B)

for (k in 1:B){
newdata<-survival.data(n,ttime, gamma0,lambda_0, tau, en)
#list(gamma_est,gamma_se, cov_prob.gamma, bw.est, SE.bw.est, cov_prob.bw, deriv.aw.est, SE.deriv.aw.est, cov_prob.deriv.aw, crate, BIC)
   
   for (s in 1:NB){   #loop for different number of basis functions;
   dfB.W=Num.Basis[s]
   result<-Estimation(newdata)
   gamma_est[,k,s]<-result[[1]]
   gamma_se[,k,s]<-result[[2]]
   cov_prob.gamma[,k,s]<-result[[3]]
   bw.est[,k,s]<-result[[4]]
   SE.bw.est[,k,s]<-result[[5]]
   cov_prob.bw[,k,s]<-result[[6]]
   deriv.aw.est[,k,s]<-result[[7]]
   SE.deriv.aw.est[,k,s]<-result[[8]]
   cov_prob.deriv.aw[,k,s]<-result[[9]]
   crate[k,s]<-result[[10]]
   BIC.funV[k,s]=result[[11]]
   dfB.WV[k,s]<-dfB.W
   } #End of s loop of different number basis functions;

### Summarize BIC results for each kth simulation;

dfB.temp<-which.min(BIC.funV[k,])            #Value of min BIC function;
dfB.WV.BIC[,k]<-dfB.WV[k, dfB.temp]         
BIC.funV.min[,k]<-BIC.funV[k, dfB.temp]      #If use min(BIC.funV[i,]), it might be NaN values;

gamma_est.BIC[,k]<-gamma_est[,k,dfB.temp] 
gamma_se.BIC[,k]<-gamma_se[,k,dfB.temp]
cov_prob.gamma.BIC[,k]<-cov_prob.gamma[,k,dfB.temp]
bw.est.BIC[,k]<-bw.est[,k,dfB.temp]
SE.bw.est.BIC[,k]<-SE.bw.est[,k,dfB.temp]
cov_prob.bw.BIC[,k]<-cov_prob.bw[,k,dfB.temp]
deriv.aw.est.BIC[,k]<-deriv.aw.est[,k,dfB.temp]
SE.deriv.aw.est.BIC[,k]<-SE.deriv.aw.est[,k,dfB.temp]
cov_prob.deriv.aw.BIC[,k]<-cov_prob.deriv.aw[,k,dfB.temp]
crate.BIC[,k] <- crate[k,dfB.temp]
}


ave_gamma_est<-rowMeans(gamma_est.BIC)
AESE_gamma<-rowMeans(gamma_se.BIC)
SSE_gamma<-sd(gamma_est.BIC)
ave_cov_prob.gamma<-rowMeans(cov_prob.gamma.BIC)
ave_bw.est<-rowMeans(bw.est.BIC)
AESE_bw<-rowMeans(SE.bw.est.BIC)
SSE_bw<-apply(bw.est.BIC,1,sd)
ave_cov_prob.bw<-rowMeans(cov_prob.bw.BIC)
ave_deriv.aw.est<-rowMeans(deriv.aw.est.BIC)
AESE_deriv.aw<-rowMeans(SE.deriv.aw.est.BIC)
SSE_deriv.aw.est<-apply(deriv.aw.est.BIC,1,sd)
ave_cov_prob.deriv.aw<-rowMeans(cov_prob.deriv.aw.BIC)
ave_crate<-rowMeans(crate.BIC)
ave_dfB.W<-rowMeans(dfB.WV.BIC)

bias_gamma<-ave_gamma_est-gamma0
bias_bw<-ave_bw.est-true.bw
bias_deriv.aw<-ave_deriv.aw.est-true_deriv_aw

cat("The seed for this simulation is             ", seed, "\n",
    "Average basis fucntions selected:           ", ave_dfB.W, "\n",
    "The sample size for this simulation is n=   ", en, "\n",
    "with censoring proportion:                  ", ave_crate, "\n",
    "Simulation times:                           ", B,"\n \n \n",
    "True gamma  value:                          ", round(gamma0,3), "\n",
    "Averaged gamma estimate:                    ", round(ave_gamma_est,3), "\n",
    "Estimated gamma bias:                       ", round(bias_gamma,3), "\n",
    "AESE of gamma:                              ", round(AESE_gamma,3), "\n",
    "SSE of gamma:                               ", round(SSE_gamma,3), "\n",
    "Coverage Probability of gamma:              ", round(ave_cov_prob.gamma,3), "\n \n \n",
    "W value:                                    ", seq(0.5,2.5,by=0.5), "\n",
    "True bw  value:                             ", round(true.bw,3), "\n",
    "Averaged bw estimate:                       ", round(ave_bw.est,3), "\n",
    "Estimated bw bias:                          ", round(bias_bw,3), "\n",
    "AESE of bw:                                 ", round(AESE_bw,3), "\n",
    "SSE of bw:                                  ", round(SSE_bw,3), "\n",
    "Coverage Probability of bw:                 ", round(ave_cov_prob.bw,3), "\n \n \n",
    "True deriv.aw  value:                       ", round(true_deriv_aw,3), "\n",
    "Averaged deriv.aw estimate:                 ", round(ave_deriv.aw.est,3), "\n",
    "Estimated deriv.aw bias:                    ", round(bias_deriv.aw,3), "\n",
    "AESE of deriv.aw:                           ", round(AESE_deriv.aw,3), "\n",
    "SSE of deriv.aw:                            ", round(SSE_deriv.aw.est,3), "\n",
    "Coverage Probability of deriv.aw:           ", round(ave_cov_prob.deriv.aw,3), "\n \n")

}  #End of j loop; 

cat("The program ended:", date(), "\n")
sink()
